package edu.brynmawr.RocketA;

import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.annotation.SuppressLint;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;

import org.json.JSONArray;
import org.json.JSONObject;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.Random;
import java.util.concurrent.Executor;
import java.util.concurrent.RejectedExecutionException;

public class FragmentLaunches extends Fragment {

    private RelativeLayout mainLayout;
    private String launcherName;
    public FragmentLaunches(JSONObject jo) {
        this.launcherName = jo.optString("name");
    }
    public FragmentLaunches(String ln) {
        this.launcherName = ln;
    }

    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState ) {
        // set up the initial layout
        RelativeLayout ll = new RelativeLayout(getContext());
        ll.setId(View.generateViewId());
        ll.setLayoutParams(new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
        mainLayout = ll;
        return ll;
    }

    /**
     * Generate a random color
     * @return the random color
     */
    private int randomColor() {
        Random r = new Random();
        return Color.rgb(r.nextInt(255), r.nextInt(255), r.nextInt(255));
    }

    public void onViewCreated(@NonNull View view, Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        /*
        TypedValue tv = new TypedValue();
        if (getContext().getTheme().resolveAttribute(android.R.attr.actionBarSize, tv, true)) {
            int actionBarHeight = TypedValue.complexToDimensionPixelSize(tv.data, getResources().getDisplayMetrics());
            view.setPadding(0, actionBarHeight, 0, 0);
        }
*/
        layoutFetching();
    }

    private void layoutFetching() {
        mainLayout.setBackgroundColor(randomColor());
        TextView tv = new TextView(getContext());
        tv.setText("Fetching Launcher List");
        tv.setGravity(Gravity.CENTER);
        ViewGroup.LayoutParams lp = new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
        mainLayout.addView(tv, lp);

        tv.setRotation(0);
        tv.animate().rotation(2880).setDuration(30000).setListener(new AnimatorListenerAdapter() {
            @Override
            public void onAnimationEnd(Animator animation) {
                super.onAnimationEnd(animation);
            }
        });
        Executor exec = new Executor() {
            @Override
            public void execute(Runnable command) {
                new Thread(command).start();
            }
        };
        try {
            exec.execute(new FragmentLaunches.GetLaunchesRunner(this, launcherName));
        }
        catch (RejectedExecutionException | NullPointerException exception) {
            Log.e("THISS", "Thread problem", exception);
        }
    }

    private void stopAndFail(String failMessage) {
        mainLayout.removeAllViews();
        TextView tv = new TextView(getContext());
        tv.setText(failMessage);
        tv.setGravity(Gravity.CENTER);
        ViewGroup.LayoutParams lp = new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
        mainLayout.addView(tv, lp);

    }

    private void gotData(ArrayList<JSONObject> aList)
    {
        mainLayout.removeAllViews();
        ListView theLv = new ListView(getContext());
        mainLayout.addView(theLv, new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
        theLv.setChoiceMode(ListView.CHOICE_MODE_SINGLE);
        theLv.setBackgroundColor(Color.rgb(240, 240, 240));
        theLv.setDivider(new ColorDrawable(0xff8B008B));
        theLv.setDividerHeight((int)(2*getResources().getDisplayMetrics().density));
        theLv.setStackFromBottom(false);
        theLv.setTranscriptMode(ListView.TRANSCRIPT_MODE_ALWAYS_SCROLL);
        ArrayAdapter<JSONObject> aa = new ArrayAdapter<JSONObject>(getContext(), 0, aList)
        {

            @NonNull
            @SuppressLint("InflateParams")
            @Override
            public View getView(int position, View convertView, @NonNull ViewGroup parent)
            {
                View view = convertView;
                if (view == null)
                {
                    view = new LinearLayout(getContext());
                }
                JSONObject msg = this.getItem(position);
                LinearLayout ll = (LinearLayout) view;
                ll.removeAllViews();
                ll.setOrientation(LinearLayout.HORIZONTAL);
                ll.setLayoutParams(new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT));
                {
                    TextView tv = new TextView(getContext());
                    tv.setText(msg.toString());
                    tv.setGravity(Gravity.LEFT | Gravity.CENTER_VERTICAL);
                    float density = getResources().getDisplayMetrics().density;
                    tv.setPadding((int)(3*density), (int)(8*density), 0, (int)(8*density));
                    ll.addView(tv, new LinearLayout.LayoutParams(0, ViewGroup.LayoutParams.WRAP_CONTENT, 1));
                }
                return view;
            }

        };
        theLv.setAdapter(aa);
    }


    private static class GetLaunchesRunner extends DownloadRunner
    {
        WeakReference<FragmentLaunches> wf;
        String vSpec;
        GetLaunchesRunner(FragmentLaunches caller, String vehSpec)
        {
            Log.i("THISS", "VSPEC " + vehSpec);
            this.vSpec=vehSpec;
            wf = new WeakReference<>(caller);
        }

        @Override
        public void run()
        {
            Log.i("THISS", "GetLaunchesRunnerr running");
            final String[][] ee = {
                    {"veh", vSpec},
            };
            // temp holder of return value
            String res = webInvokeJSON("http://comet.cs.brynmawr.edu/~gtowell/380/rocket/launchesJ.php", ee);
            if (res ==null)
            {
                // communication failure!!! so should return false
                new Handler(Looper.getMainLooper()).post(new Runnable()
                {
                    @Override
                    public void run()
                    {
                        FragmentLaunches fl = wf.get();
                        if (fl!=null)
                            fl.stopAndFail("Failed to get information from server");
                    }
                });
                return;
            }
            try
            {
                Log.i("THISS", "MatchesAAA " + res);
                JSONArray jo = new JSONArray(res);
                //Log.i("THISS", "MatchesAAA " + jo);
                new Handler(Looper.getMainLooper()).post(new Runnable()
                {
                    @Override
                    public void run()
                    {
                        FragmentLaunches fl = wf.get();
                        if (fl!=null) {
                            ArrayList<JSONObject> al = new ArrayList<>();
                            for (int i = 0; i < jo.length(); i++) {
                                try {
                                    al.add(jo.getJSONObject(i));
                                } catch (Exception e) {

                                }
                            }
                            fl.gotData(al);
                        }
                    }
                });
                return;
            }
            catch (Exception eee)
            {
                Log.e("THISS", "Problme", eee);
            }
            new Handler(Looper.getMainLooper()).post(new Runnable()
            {
                @Override
                public void run()
                {
                    FragmentLaunches fl = wf.get();
                    if (fl!=null)
                        fl.stopAndFail("Something weird happened");
                }
            });
        }
    }


}
